////////////////////////////////////////////////////////////////////////////////////
/// @file CObjet.cpp
/// @author DGI
/// @date 2005-06-15
/// @version 1.0
///
/// @addtogroup chargeur3ds Chargeur 3DS
/// @{
////////////////////////////////////////////////////////////////////////////////////

#ifdef WIN32
#include <windows.h>
#endif

#include <GL/gl.h>

#include "CObjet.h"

namespace chargeur3ds {


   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::CObjet()
   ///
   /// Constructeur par dfaut
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   CObjet::CObjet()
   {
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::~CObjet()
   ///
   /// Destructeur par dfaut
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   CObjet::~CObjet()
   {
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::mettreAEchelle ( float f)
   ///
   /// Fonction de mise  l'chelle de l'objet
   ///
   /// @param[in] f : Facteur de mise  l'chelle
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::mettreAEchelle ( float f) 
   {
       for ( unsigned int i = 0; i < vvert.size(); i++) {
           vvert[i].pos *= f;
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn double CObjet::calculerSphereEnglobante() const
   ///
   /// Calcule et retourne une sphre englobant l'objet
   ///
   /// @return Le rayon de la sphre englobante.
   ///
   ////////////////////////////////////////////////////////////////////////
   double CObjet::calculerSphereEnglobante() const
   {
      if (vvert.size() == 0) {
         // Aucun vertex
         return 0;
      }
      else {
         double rayon = vvert[0].pos.norme();

         for ( unsigned int i = 1; i < vvert.size(); i++) {
            double r = vvert[i].pos.norme();

            if (rayon < r)
               rayon = r;
           }

         return rayon;
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn void CObjet::calculerCylindreEnglobant(double& rayon, double& bas, double& haut) const
   ///
   /// Calcule et retourne un cylindre englobant l'objet
   ///
   /// @param[out] rayon : Le rayon du cylindre.
   /// @param[out] bas   : La hauteur du bas du cylindre.
   /// @param[out] haut  : La hauteur du haut du cylindre.
   ///
   /// @return Aucune.
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::calculerCylindreEnglobant(double& rayon, double& bas, double& haut) const
   {
      if (vvert.size() == 0) {
         // Aucun vertex
         rayon = bas = haut = 0;
      }
      else {
         rayon = vvert[0].pos.convertir<2>().norme();
         bas = haut = vvert[0].pos[2];

         for ( unsigned int i = 1; i < vvert.size(); i++) {
            double z = vvert[i].pos[2];
            double r = vvert[i].pos.convertir<2>().norme();

            if (rayon < r)
               rayon = r;
            if (bas > z)
               bas = z;
            if (haut < z)
               haut = z;
           }
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn void CObjet::calculerBoiteEnglobante(Vecteur3& coinMin, Vecteur3& coinMax) const
   ///
   /// Calcule et retourne une bote englobante de l'objet.
   ///
   /// @param[out] coinMin : Le coin de coordonnes minimales de la bote englobante.
   /// @param[out] coinMax : Le coin de coordonnes maximales de la bote englobante.
   ///
   /// @return Aucune.
   ///
   ///////////////////////////////////////////////////////////////////////
   void CObjet::calculerBoiteEnglobante(Vecteur3& coinMin, Vecteur3& coinMax) const
   {
      if (vvert.size() == 0) {
         // Aucun vertex
         coinMin = coinMax = Vecteur3(0);
      }
      else {
         coinMin = coinMax = vvert[0].pos;

         for ( unsigned int i = 1; i < vvert.size(); i++) {
            const Vecteur3& v = vvert[i].pos;

            // Comparaison avec le coin minimal
            if (coinMin[0] > v[0])
               coinMin[0] = v[0];
            if (coinMin[1] > v[1])
               coinMin[1] = v[1];
            if (coinMin[2] > v[2])
               coinMin[2] = v[2];

            // Comparaison avec le coin maximal
            if (coinMax[0] < v[0])
               coinMax[0] = v[0];
            if (coinMax[1] < v[1])
               coinMax[1] = v[1];
            if (coinMax[2] < v[2])
               coinMax[2] = v[2];
         }
      }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::dessiner() const
   ///
   /// Description : Dessine l'objet
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::dessiner() const
   {
       //        vecteurMateriaux_[obj.mat_id].Activate();
       ptrMateriel->Activate();

       glBegin ( GL_TRIANGLES);
       for ( unsigned int j = 0; j < vface.size(); j++)     {
           Vecteur3 fnormal = vface[j].normal;
           glNormal3d ( fnormal[0], fnormal[1], fnormal[2]);
           for ( int k = 0; k < 3; k++) {
               int vid=vface[j].id[k];
               int tcid=vface[j].tid[k];

               Vecteur3 vnormal=vface[j].vnormal[k];

               //                    if  ( dot ( vnormal,fnormal)>.9f)
               {
                   glNormal3d ( vnormal[0],vnormal[1],vnormal[2]);
               }

               Vecteur2f tcoord = vtc[tcid].tc;
               glTexCoord2f ( tcoord[0],tcoord[1]);
               Vecteur3 pos = vvert[vid].pos;
               glVertex3d ( pos[0],pos[1],pos[2]);
           }
       }
       glEnd();
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn Vecteur3 CObjet::lireMassCenter() const
   ///
   /// Lecture du centre de masse de l'objet
   ///
   /// @return Centre de masse de l'objet
   ///
   ////////////////////////////////////////////////////////////////////////
   Vecteur3 CObjet::lireMassCenter() const
   {
       Vecteur3 r ( 0.0, 0.0, 0.0 );
       for ( unsigned int i = 0; i < vvert.size(); i++) {
           r += vvert[i].pos;
       }
       r = r / ( float)vvert.size();
       return r;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::Translate ( Vecteur3 delta)
   ///
   /// Translation de l'objet
   ///
   /// @param[in] delta : Vecteur de translation
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::Translate ( Vecteur3 delta)     
   {
       for ( unsigned int i = 0; i < vvert.size(); i++) {
           vvert[i].pos += delta;
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::tourner( float angle , const Vecteur3& axe )
   ///
   /// Rotation de l'objet
   ///
   /// @param[in] angle : Angle de rotation, en radians
   /// @param[in] axe   : Axe de rotation (unitaire)
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::tourner( float angle , const Vecteur3& axe )
   {
      for ( unsigned int i = 0; i < vvert.size(); i++) {
         vvert[i].pos = vvert[i].pos.tourner(angle, axe);
      }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn CObjet::importer ( ifstream& in)
   ///
   /// Lecture d'un objet  partir d'un stream
   ///
   /// @param[in] in : Flux de lecture
   ///
   /// @return Aucune
   ///
   ////////////////////////////////////////////////////////////////////////
   void CObjet::importer ( ifstream& in)
   {
       string r;
       in>>r;//mat-id
       in>>mat_id;

       in>>r;//name;

       in>>name;

       int n_vert;

       in>>r;// n_vert
       in>>n_vert;

       in>>r;// position
       in>>positionPivot;

       for ( int i = 0; i < n_vert; i++) {
           vvert.push_back ( CVertex());
           vvert.back().importer ( in);
       }

       int n_tvert;
       in>>r;
       in>>n_tvert;


       for ( int i = 0; i < n_tvert; i++) {
           vtc.push_back ( CCoordonnee());
           vtc.back().importer ( in);
       }

       int n_face;
       in>>r;
       in>>n_face;


       for ( int i = 0; i < n_face; i++) {
           vface.push_back ( CFace());
           vface.back().importer ( in);
       }
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn operator< ( CEcritureFichierBinaire& out, const CObjet& sobj)
   ///
   /// Surcharge de l'oprateur <
   ///
   /// @param[out] out : Fichier binaire d'criture
   /// @param[in] sobj : Objet  enregistrer dans le fichier
   ///
   /// @return Flux du fichier d'criture
   ///
   ////////////////////////////////////////////////////////////////////////
   CEcritureFichierBinaire& operator< ( CEcritureFichierBinaire& out, const CObjet& sobj)
   {
       out<sobj.mat_id;
       out<sobj.name;
       out<sobj.positionPivot;
       out< int ( sobj.vvert.size());
       for ( unsigned int i = 0; i <sobj.vvert.size(); i++) {
           out<sobj.vvert[i];
       }
       out<int ( sobj.vtc.size());
       for ( unsigned int i = 0; i <sobj.vtc.size(); i++) {
           out<sobj.vtc[i];
       }
       out<int ( sobj.vface.size());
       for ( unsigned int i = 0; i < sobj.vface.size(); i++) {
           out<sobj.vface[i];
       }
       return out;
   }

   ////////////////////////////////////////////////////////////////////////
   ///
   /// @fn operator> ( CLectureFichierBinaire& in, CObjet& sobj)
   ///
   /// Surcharge de l'oprateur >
   ///
   /// @param[in]  in   : Fichier bianire de lecture
   /// @param[out] sobj : Objet dans lequel l'information est lue
   ///
   /// @return Flux de lecture
   ///
   ////////////////////////////////////////////////////////////////////////
   CLectureFichierBinaire& operator> ( CLectureFichierBinaire& in, CObjet& sobj)
   {
       // Lire les informations initiales
       in>sobj.mat_id;
       in>sobj.name;
       in>sobj.positionPivot;

       // Lire les vertex
       unsigned int size;
       in>size;
       sobj.vvert.resize ( size);
       for ( unsigned int i = 0 ; i < sobj.vvert.size(); i++){
           in>sobj.vvert[i];
       }
       // Lire les coordones des points
       in>size;
       sobj.vtc.resize ( size);
       for ( unsigned int i = 0; i < sobj.vtc.size(); i++)    {
           in>sobj.vtc[i];
       }

       // Lire les faces
       in>size;
       sobj.vface.resize ( size);
       for ( unsigned int i = 0; i < sobj.vface.size(); i++) {
           in>sobj.vface[i];
       }
       return in;
   }

} // Fin de l'espace de nom chargeur3ds


////////////////////////////////////////////////////////////////////////////////////
/// @}
////////////////////////////////////////////////////////////////////////////////////
